@@ -21,3 +21,22 @@ MIDONET_NET_TYPES = [TYPE_MIDONET, TYPE_UPLINK]
 
 # Midonet VIF TYPE
 VIF_TYPE_MIDONET = 'midonet'
+
+# Neutron well-known service type constants:
+GATEWAY_DEVICE = "GATEWAY_DEVICE"
+LOGGING_RESOURCE = "LOGGING_RESOURCE"
+
+# for Midonet L2 Gateway
+MAX_VXLAN_VNI = 16777215
+MIDONET_L2GW_PROVIDER = "midonet"
+
+# (Kengo) We define constants until
+# upstream will deal with router status.
+ROUTER_STATUS_ACTIVE = "ACTIVE"
+ROUTER_STATUS_ERROR = "ERROR"
+
+# for resource name on callback method
+MIDONET_NETWORK = "midonet_network"
+
+# for bgp dynamic router extension
+LOGICAL_ROUTER = 'logical_router'
