@@ -15,7 +15,6 @@
 
 from neutron_lib import exceptions as exc
 from oslo_log import log
-import six
 
 from neutron.plugins.ml2 import driver_api as api
 
@@ -25,7 +24,7 @@ from midonet.neutron.common import constants as const
 LOG = log.getLogger(__name__)
 
 
-class MidonetTypeDriver(api.TypeDriver):
+class MidonetTypeDriver(api.ML2TypeDriver):
     """Type driver for Midonet networks
 
     This type driver differentiates midonet networks from other types.
@@ -44,18 +43,18 @@ class MidonetTypeDriver(api.TypeDriver):
         return False
 
     def validate_provider_segment(self, segment):
-        for key, value in six.iteritems(segment):
+        for key, value in segment.items():
             if value and key != api.NETWORK_TYPE:
                 msg = _("%s prohibited for midonet provider network") % key
                 raise exc.InvalidInput(error_message=msg)
 
-    def reserve_provider_segment(self, session, segment):
+    def reserve_provider_segment(self, context, segment):
         return segment
 
-    def allocate_tenant_segment(self, session):
+    def allocate_tenant_segment(self, context):
         return {api.NETWORK_TYPE: const.TYPE_MIDONET}
 
-    def release_segment(self, session, segment):
+    def release_segment(self, context, segment):
         pass
 
     def get_mtu(self, physical):
