@@ -15,23 +15,26 @@
 
 from neutron_lib import constants as n_const
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import directory
 
 from midonet.neutron._i18n import _, _LE, _LW
 from midonet.neutron.common import utils as c_utils
-from midonet.neutron.db import agent_membership_db as am_db
 from midonet.neutron.db import port_binding_db as pb_db
 from midonet.neutron.db import provider_network_db as pnet_db
+from midonet.neutron.midonet_v2 import managers
 from midonet.neutron import plugin
+from midonet.neutron.services.qos import driver as qos_driver
+
+from neutron.api.v2 import attributes
 from neutron.db import allowedaddresspairs_db as addr_pair_db
 from neutron.db import api as db_api
+from neutron.db import db_base_plugin_v2
 from neutron.db import portsecurity_db as ps_db
 from neutron.extensions import allowedaddresspairs as addr_pair
 from neutron.extensions import extra_dhcp_opt as edo_ext
 from neutron.extensions import portsecurity as psec
 from neutron.extensions import providernet as pnet
 from neutron.extensions import securitygroup as ext_sg
-from neutron import manager
-from neutron.plugins.common import constants as service_constants
 from oslo_db import exception as oslo_db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -42,7 +45,6 @@ LOG = logging.getLogger(__name__)
 
 class MidonetPluginV2(plugin.MidonetMixinBase,
                       addr_pair_db.AllowedAddressPairsMixin,
-                      am_db.AgentMembershipDbMixin,
                       pnet_db.MidonetProviderNetworkMixin,
                       pb_db.MidonetPortBindingMixin,
                       ps_db.PortSecurityDbMixin):
@@ -53,9 +55,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
     # alternative metadata proxy which doesn't require Neutron agents.
     # NOTE(yamamoto): While the order in this list doesn't matter
     # functionality-wise, it's alphabetically sorted for easier comparison.
-    supported_extension_aliases = [
+    _base_supported_extension_aliases = [
         'agent',
-        'agent-membership',
         'allowed-address-pairs',
         'binding',
         'dhcp_agent_scheduler',
@@ -68,14 +69,26 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         'subnet_allocation',
     ]
 
+    @property
+    def supported_extension_aliases(self):
+        if not hasattr(self, '_aliases'):
+            aliases = self._base_supported_extension_aliases
+            aliases += self.extension_manager.extension_aliases()
+            self._aliases = aliases
+        return self._aliases
+
     __native_bulk_support = True
     __native_pagination_support = True
     __native_sorting_support = True
 
     def __init__(self):
+        self.extension_manager = managers.ExtensionManager()
+        self.extension_manager.initialize()
         super(MidonetPluginV2, self).__init__()
         self.client.initialize()
+        qos_driver.register()
 
+    @db_api.retry_if_session_inactive()
     def create_network(self, context, network):
         LOG.debug('MidonetPluginV2.create_network called: network=%r', network)
 
@@ -88,6 +101,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             net_db = self.create_network_db(context, network)
             net = self._make_network_dict(net_db, process_extensions=False,
                                           context=context)
+            self.extension_manager.process_create_network(context, net_data,
+                                                          net)
             net_data['id'] = net['id']
             self._process_l3_create(context, net, net_data)
             self._create_provider_network(context, net_data)
@@ -142,6 +157,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
 
         return [self._fields(net, fields) for net in nets]
 
+    @db_api.retry_if_session_inactive()
     def update_network(self, context, id, network):
         LOG.debug("MidonetPluginV2.update_network called: id=%(id)r, "
                   "network=%(network)r", {'id': id, 'network': network})
@@ -153,6 +169,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         with context.session.begin(subtransactions=True):
             net = super(MidonetPluginV2, self).update_network(
                 context, id, network)
+            self.extension_manager.process_update_network(context, net_data,
+                                                          net)
             self._process_l3_update(context, net, network['network'])
             self._extend_provider_network_dict(context, net)
             if psec.PORTSECURITY in network['network']:
@@ -170,6 +188,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.update_network exiting: net=%r", net)
         return net
 
+    @db_api.retry_if_session_inactive()
     def delete_network(self, context, id):
         LOG.debug("MidonetPluginV2.delete_network called: id=%r", id)
 
@@ -192,11 +211,14 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
 
         LOG.debug("MidonetPluginV2.delete_network exiting: id=%r", id)
 
+    @db_api.retry_if_session_inactive()
     def create_subnet(self, context, subnet):
         LOG.debug("MidonetPluginV2.create_subnet called: subnet=%r", subnet)
 
         with context.session.begin(subtransactions=True):
             s = super(MidonetPluginV2, self).create_subnet(context, subnet)
+            self.extension_manager.process_create_subnet(context,
+                subnet['subnet'], s)
             self.client.create_subnet_precommit(context, s)
 
         try:
@@ -213,6 +235,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.create_subnet exiting: subnet=%r", s)
         return s
 
+    @db_api.retry_if_session_inactive()
     def delete_subnet(self, context, id):
         LOG.debug("MidonetPluginV2.delete_subnet called: id=%s", id)
 
@@ -224,11 +247,14 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
 
         LOG.debug("MidonetPluginV2.delete_subnet exiting")
 
+    @db_api.retry_if_session_inactive()
     def update_subnet(self, context, id, subnet):
         LOG.debug("MidonetPluginV2.update_subnet called: id=%s", id)
 
         with context.session.begin(subtransactions=True):
             s = super(MidonetPluginV2, self).update_subnet(context, id, subnet)
+            self.extension_manager.process_update_subnet(context,
+                subnet['subnet'], s)
             # NOTE(yamamoto): Retrieve the db object to get the correct
             # revision
             context.session.flush()
@@ -240,6 +266,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.update_subnet exiting: subnet=%r", s)
         return s
 
+    @db_api.retry_if_session_inactive()
     def create_port(self, context, port):
         LOG.debug("MidonetPluginV2.create_port called: port=%r", port)
 
@@ -254,6 +281,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             # Create a Neutron port
             port_db = self.create_port_db(context, port)
             new_port = self._make_port_dict(port_db, process_extensions=False)
+            self.extension_manager.process_create_port(context, port_data,
+                                                       new_port)
 
             # Do not create a gateway port if it has no IP address assigned as
             # MidoNet does not yet handle this case.
@@ -327,13 +356,13 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.create_port exiting: port=%r", new_port)
         return new_port
 
+    @db_api.retry_if_session_inactive()
     def delete_port(self, context, id, l3_port_check=True):
         LOG.debug("MidonetPluginV2.delete_port called: id=%(id)s "
                   "l3_port_check=%(l3_port_check)r",
                   {'id': id, 'l3_port_check': l3_port_check})
 
-        l3plugin = manager.NeutronManager.get_service_plugins().get(
-            service_constants.L3_ROUTER_NAT)
+        l3plugin = directory.get_plugin(n_const.L3)
 
         # if needed, check to see if this is a port owned by
         # and l3-router.  If so, we should prevent deletion.
@@ -351,6 +380,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
 
         LOG.debug("MidonetPluginV2.delete_port exiting: id=%r", id)
 
+    @db_api.retry_if_session_inactive()
     def update_port(self, context, id, port):
         LOG.debug("MidonetPluginV2.update_port called: id=%(id)s "
                   "port=%(port)r", {'id': id, 'port': port})
@@ -361,6 +391,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             original_port = super(MidonetPluginV2, self).get_port(context, id)
             p = super(MidonetPluginV2, self).update_port(context, id, port)
             c_utils.check_update_port(original_port, p)
+            self.extension_manager.process_update_port(context, port['port'],
+                                                       p)
 
             has_sg = self._check_update_has_security_groups(port)
             delete_sg = self._check_update_deletes_security_groups(port)
@@ -394,21 +426,22 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             p = self.get_port(context, id)
             self.client.update_port_precommit(context, id, p)
 
-        # Set status along admin_state_up to update.
-        if p['admin_state_up']:
-            update_status = n_const.PORT_STATUS_ACTIVE
-        else:
-            update_status = n_const.PORT_STATUS_DOWN
-
         try:
             self.client.update_port_postcommit(id, p)
-            data = {'port': {'status': update_status}}
-            new_port = super(MidonetPluginV2, self).update_port(
-                context, id, data)
-            # prevent binding:profile information from lacking
-            p['status'] = new_port['status']
-            if 'revision_number' in new_port:
-                p['revision_number'] = new_port['revision_number']
+            if p['status'] != n_const.PORT_STATUS_NOTAPPLICABLE:
+                # Set status along admin_state_up to update.
+                if p['admin_state_up']:
+                    update_status = n_const.PORT_STATUS_ACTIVE
+                else:
+                    update_status = n_const.PORT_STATUS_DOWN
+
+                data = {'port': {'status': update_status}}
+                new_port = super(MidonetPluginV2, self).update_port(
+                    context, id, data)
+                # prevent binding:profile information from lacking
+                p['status'] = new_port['status']
+                if 'revision_number' in new_port:
+                    p['revision_number'] = new_port['revision_number']
         except Exception as ex:
             with excutils.save_and_reraise_exception():
                 LOG.error(_LE("Failed to update a port %(port_id)s in "
@@ -422,6 +455,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.update_port exiting: p=%r", p)
         return p
 
+    @db_api.retry_if_session_inactive()
     def create_security_group(self, context, security_group, default_sg=False):
         LOG.debug("MidonetPluginV2.create_security_group called: "
                   "security_group=%(security_group)s "
@@ -455,6 +489,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.create_security_group exiting: sg=%r", sg)
         return sg
 
+    @db_api.retry_if_session_inactive()
     def delete_security_group(self, context, id):
         LOG.debug("MidonetPluginV2.delete_security_group called: id=%s", id)
 
@@ -473,6 +508,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
 
         LOG.debug("MidonetPluginV2.delete_security_group exiting: id=%r", id)
 
+    @db_api.retry_if_session_inactive()
     def create_security_group_rule(self, context, security_group_rule):
         LOG.debug("MidonetPluginV2.create_security_group_rule called: "
                   "security_group_rule=%(security_group_rule)r",
@@ -499,6 +535,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                   "rule=%r", rule)
         return rule
 
+    @db_api.retry_if_session_inactive()
     def create_security_group_rule_bulk(self, context, security_group_rules):
         LOG.debug("MidonetPluginV2.create_security_group_rule_bulk called: "
                   "security_group_rules=%(security_group_rules)r",
@@ -524,6 +561,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                   "rules=%r", rules)
         return rules
 
+    @db_api.retry_if_session_inactive()
     def delete_security_group_rule(self, context, sg_rule_id):
         LOG.debug("MidonetPluginV2.delete_security_group_rule called: "
                   "sg_rule_id=%s", sg_rule_id)
@@ -539,80 +577,25 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         LOG.debug("MidonetPluginV2.delete_security_group_rule exiting: id=%r",
                   sg_rule_id)
 
-    def create_agent_membership(self, context, agent_membership):
-        LOG.debug("MidonetPluginV2.create_agent_membership called: "
-                  " %(agent_membership)r",
-                  {'agent_membership': agent_membership})
+    def _midonet_v2_extend_network_dict(self, result, netdb):
+        session = db_api.get_session()
+        with session.begin(subtransactions=True):
+            self.extension_manager.extend_network_dict(session, netdb, result)
 
-        with context.session.begin(subtransactions=True):
-            am = super(MidonetPluginV2, self).create_agent_membership(
-                context, agent_membership)
-            self.client.create_agent_membership_precommit(context, am)
+    def _midonet_v2_extend_port_dict(self, result, portdb):
+        session = db_api.get_session()
+        with session.begin(subtransactions=True):
+            self.extension_manager.extend_port_dict(session, portdb, result)
 
-        try:
-            self.client.create_agent_membership_postcommit(am)
-        except Exception as ex:
-            with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to create agent membership. am: %(am)r, "
-                              "error: %(err)s"), {'am': am, 'err': ex})
-                try:
-                    self.delete_agent_membership(context, am['id'])
-                except Exception:
-                    LOG.exception(_LE("Failed to delete "
-                                      "an agent membership %s"), am['id'])
+    def _midonet_v2_extend_subnet_dict(self, result, subnetdb):
+        session = db_api.get_session()
+        with session.begin(subtransactions=True):
+            self.extension_manager.extend_subnet_dict(
+                session, subnetdb, result)
 
-        LOG.debug("MidonetPluginV2.create_agent_membership exiting: "
-                  "%(agent_membership)r", {'agent_membership': am})
-        return am
-
-    def get_agent_membership(self, context, id, filters=None, fields=None):
-        LOG.debug("MidonetPluginV2.get_agent_membership called: id=%(id)r",
-                  {'id': id})
-
-        with context.session.begin(subtransactions=True):
-            am = super(MidonetPluginV2, self).get_agent_membership(context, id)
-
-        LOG.debug("MidonetPluginV2.get_agent_membership exiting: id=%(id)r, "
-                  "agent_membership=%(agent_membership)r",
-                  {'id': id, 'agent_membership': am})
-        return am
-
-    def get_agent_memberships(self, context, filters=None, fields=None,
-                              sorts=None, limit=None, marker=None,
-                              page_reverse=False):
-        LOG.debug("MidonetPluginV2.get_agent_memberships called")
-
-        with context.session.begin(subtransactions=True):
-            ams = super(MidonetPluginV2, self).get_agent_memberships(
-                context, filters, fields, sorts, limit, marker, page_reverse)
-
-        LOG.debug("MidonetPluginV2.get_agent_memberships exiting")
-        return ams
-
-    def delete_agent_membership(self, context, id):
-        LOG.debug("MidonetPluginV2.delete_agent_membership called: %(id)r",
-                  {'id': id})
-
-        with context.session.begin(subtransactions=True):
-            super(MidonetPluginV2, self).delete_agent_membership(context, id)
-            self.client.delete_agent_membership_precommit(context, id)
-
-        self.client.delete_agent_membership_postcommit(id)
-
-        LOG.debug("MidonetPluginV2.delete_agent_membership exiting: %(id)r",
-                  {'id': id})
-
-    def get_agents(self, context, filters=None, fields=None):
-        LOG.debug("MidonetPluginV2.get_agents called")
-
-        agents = super(MidonetPluginV2, self).get_agents(context, filters,
-                                                         fields)
-        return agents + self.client.get_agents()
-
-    def get_agent(self, context, id, fields=None):
-        LOG.debug("MidonetPluginV2.get_agent called: %(id)r", {'id': id})
-
-        agent = self.client.get_agent(id)
-        if not agent:
-            agent = super(MidonetPluginV2, self).get_agent(context, id, fields)
-        return agent
+    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
+               attributes.NETWORKS, ['_midonet_v2_extend_network_dict'])
+    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
+               attributes.PORTS, ['_midonet_v2_extend_port_dict'])
+    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
+               attributes.SUBNETS, ['midonet_v2_extend_subnet_dict'])
